unit frmmain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls,
  PrintersDlgs;

type

  { TMainForm }

  TMainForm = class(TForm)
    BPrint: TButton;
    BPageSetup: TButton;
    BPrinterSetup: TButton;
    CBBorder: TCheckBox;
    Image1: TImage;
    MText: TMemo;
    PSDDemo: TPrinterSetupDialog;
    PSDemo: TPageSetupDialog;
    PDDemo: TPrintDialog;
    procedure BPageSetupClick(Sender: TObject);
    procedure BPrintClick(Sender: TObject);
    procedure BPrinterSetupClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    procedure GetPageRect(out PageRect: TRect);
    procedure PrintMemo(DrawBorder: Boolean);
  public

  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

uses Printers;

{ TMainForm }

procedure TMainForm.BPageSetupClick(Sender: TObject);
begin
  PSDemo.Execute;
end;


Function MMToPixel(aMM : Double; aRes : integer) : Integer;

begin
  Result:=Round((amm/25.4) * aRes)
end;

Function CMToPixel(aCM : Double; aRes : integer) : Integer;

begin
  Result:=Round((aCm/2.54) *aRes);
end;

Function InchToPixel(aInch : Double; aRes : integer) : Integer;

begin
  Result:=Round(aInch * aRes);
end;


procedure TMainForm.GetPageRect(Out PageRect : TRect);

  Function ToPixels(aUnit : Double; ares : Integer) : Integer;

  begin
    if PSDemo.Units in [pmDefault,pmMillimeters] then
      // 1/100 of a mm
      Result:=MMtoPixel(aUnit/100,Ares)
    else
      // 1/100 of an inch
      Result:=InchToPixel(aUnit/100,Ares)
  end;


Var
  LM,RM,TM,BM,H,W : Integer;
  WR : TRect;

begin
  LM:=ToPixels(PSDemo.MarginLeft,Printer.XDPI);
  RM:=ToPixels(PSDemo.MarginRight,Printer.XDPI);
  TM:=ToPixels(PSDemo.MarginTop,Printer.YDPI);
  BM:=ToPixels(PSDemo.MarginBottom,Printer.YDPI);
  // Work rect
  WR:=Printer.PaperSize.PaperRect.WorkRect;
  W:=Printer.PaperSize.Width-WR.Left;
  H:=Printer.PaperSize.Height-WR.Top;
  PageRect:=Rect(LM-WR.Left,TM-WR.Top,
                 W-RM,H-BM);
end;

procedure TMainForm.PrintMemo(DrawBorder : Boolean);

Var
  PageRect : TRect;
  I,D,T,L,Y : Integer;

begin
  Printer.BeginDoc;
  Printer.Title:='Print demo page';
  Printer.PaperSize.PaperName:='A4';
//  Printer.NewPage;
  with Printer.Canvas do
    begin
    Pen.Color:=clBlack;
    Pen.Style:=psSolid;
    Brush.Style:=bsSolid;
    Brush.Color:=clNone;
    Font.Name:= 'Dejavu Sans';
    Font.Size:=10;
    GetPageRect(PageRect);
    if DrawBorder then
      Rectangle(PageRect);
    L:=PageRect.Left+CMToPixel(0.5,Printer.XDPI);
    T:=PageRect.Left+CMToPixel(0.5,Printer.YDPI);
    D:=Round(TextHeight('M')*1.2);
    I:=0;
    Y:=T;
    While (I<MText.Lines.Count) do
      begin
      TextOut(L,Y,MText.Lines[i]);
      Y:=Y+D;
      Inc(I);
      If (Y>=PageRect.Bottom-(D div 2)) and (I<MText.Lines.Count) then
        begin
        Printer.NewPage;
        Y:=T;
        if DrawBorder then
          Rectangle(PageRect);
        end;
      end;
    end;
  Printer.EndDoc;
end;

procedure TMainForm.BPrintClick(Sender: TObject);

begin
  If PDDemo.Execute then
    PrintMemo(CBBorder.Checked);
end;

procedure TMainForm.BPrinterSetupClick(Sender: TObject);
begin
  PSDDemo.Execute;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Printer.SetPrinter(Printer.Printers[0]);
  MText.Lines.AddStrings(Printer.Fonts);
end;

end.

